import VisibilityIcon from "@mui/icons-material/Visibility";
import {
  Button,
  IconButton,
  Paper,
  Table,
  TableBody,
  TableCell,
  TableContainer,
  TableFooter,
  TableHead,
  TableRow,
  Tooltip,
} from "@mui/material";
import { filesize } from "filesize";
import React from "react";
import { useNavigate } from "react-router-dom";
import { VMApi, VMInfo, VMState } from "../api/VMApi";
import { AsyncWidget } from "../widgets/AsyncWidget";
import { RouterLink } from "../widgets/RouterLink";
import { VirtWebRouteContainer } from "../widgets/VirtWebRouteContainer";
import { VMStatusWidget } from "../widgets/vms/VMStatusWidget";

export function VMListRoute(): React.ReactElement {
  const [list, setList] = React.useState<VMInfo[] | undefined>();

  const loadKey = React.useRef(1);

  const load = async () => {
    setList(await VMApi.GetList());
  };

  const reload = () => {
    loadKey.current += 1;
    setList(undefined);
  };

  return (
    <AsyncWidget
      loadKey={loadKey.current}
      errMsg="Failed to load Virtual Machines list!"
      load={load}
      ready={list !== undefined}
      build={() => (
        <VirtWebRouteContainer
          label="Virtual Machines"
          actions={
            <>
              <RouterLink to="/vms/new">
                <Button>New</Button>
              </RouterLink>
            </>
          }
        >
          <VMListWidget list={list!} onReload={reload} />
        </VirtWebRouteContainer>
      )}
    />
  );
}

function VMListWidget(p: {
  list: VMInfo[];
  onReload: () => void;
}): React.ReactElement {
  const navigate = useNavigate();

  const [runningVMs, setRunningVMs] = React.useState<Set<string>>(new Set());

  const updateVMState = (v: VMInfo, s: VMState) => {
    const running = s !== "Shutoff";
    if (runningVMs.has(v.name) === running) {
      return;
    }

    if (running) runningVMs.add(v.name);
    else runningVMs.delete(v.name);

    setRunningVMs(new Set([...runningVMs]));
  };

  return (
    <TableContainer component={Paper}>
      <Table>
        <TableHead>
          <TableRow>
            <TableCell>Name</TableCell>
            <TableCell>Description</TableCell>
            <TableCell>Memory</TableCell>
            <TableCell>vCPU</TableCell>
            <TableCell>Status</TableCell>
            <TableCell>Actions</TableCell>
          </TableRow>
        </TableHead>
        <TableBody>
          {p.list.map((row) => (
            <TableRow
              hover
              key={row.name}
              sx={{ "&:last-child td, &:last-child th": { border: 0 } }}
              onDoubleClick={() => navigate(row.ViewURL)}
            >
              <TableCell component="th" scope="row">
                {row.name}
              </TableCell>
              <TableCell>{row.description ?? ""}</TableCell>
              <TableCell>{vmMemoryToHuman(row.memory)}</TableCell>
              <TableCell>{row.number_vcpu}</TableCell>
              <TableCell>
                <VMStatusWidget
                  vm={row}
                  onChange={(s) => updateVMState(row, s)}
                />
              </TableCell>
              <TableCell>
                <Tooltip title="View this VM">
                  <RouterLink to={row.ViewURL}>
                    <IconButton>
                      <VisibilityIcon />
                    </IconButton>
                  </RouterLink>
                </Tooltip>
              </TableCell>
            </TableRow>
          ))}
        </TableBody>
        <TableFooter>
          <TableRow>
            <TableCell></TableCell>
            <TableCell></TableCell>
            <TableCell>
              {vmMemoryToHuman(
                p.list
                  .filter((v) => runningVMs.has(v.name))
                  .reduce((s, v) => s + v.memory, 0)
              )}
              {" / "}
              {vmMemoryToHuman(p.list.reduce((s, v) => s + v.memory, 0))}
            </TableCell>
            <TableCell>
              {p.list
                .filter((v) => runningVMs.has(v.name))
                .reduce((s, v) => s + v.number_vcpu, 0)}
              {" / "}
              {p.list.reduce((s, v) => s + v.number_vcpu, 0)}
            </TableCell>
            <TableCell></TableCell>
            <TableCell></TableCell>
          </TableRow>
        </TableFooter>
      </Table>
    </TableContainer>
  );
}

function vmMemoryToHuman(size: number): string {
  return filesize(size * 1000 * 1000);
}
